@@ -36,7 +36,7 @@ from PyQt5.QtWidgets import QLabel
 from qutebrowser.config import config, configexc
 from qutebrowser.keyinput import modeman, modeparsers, basekeyparser
 from qutebrowser.browser import webelem, history
-from qutebrowser.commands import userscripts, runners
+from qutebrowser.commands import runners
 from qutebrowser.api import cmdutils
 from qutebrowser.utils import usertypes, log, qtutils, message, objreg, utils
 if TYPE_CHECKING:
@@ -92,6 +92,8 @@ class HintLabel(QLabel):
         self._context = context
         self.elem = elem
 
+        self.setTextFormat(Qt.RichText)
+
         # Make sure we can style the background via a style sheet, and we don't
         # get any extra text indent from Qt.
         # The real stylesheet lives in mainwindow.py for performance reasons..
@@ -270,7 +272,7 @@ class HintActions:
         msg = "Yanked URL to {}: {}".format(
             "primary selection" if sel else "clipboard",
             urlstr)
-        message.info(msg)
+        message.info(msg, replace='rapid-hints' if context.rapid else None)
 
     def run_cmd(self, url: QUrl, context: HintContext) -> None:
         """Run the command based on a hint URL."""
@@ -318,6 +320,9 @@ class HintActions:
             elem: The QWebElement to use in the userscript.
             context: The HintContext to use.
         """
+        # lazy import to avoid circular import issues
+        from qutebrowser.commands import userscripts
+
         cmd = context.args[0]
         args = context.args[1:]
         env = {
@@ -1000,7 +1005,7 @@ class HintManager(QObject):
             self._context.first_run = False
 
     @cmdutils.register(instance='hintmanager', scope='window',
-                       modes=[usertypes.KeyMode.hint], deprecated_name='follow-hint')
+                       modes=[usertypes.KeyMode.hint])
     def hint_follow(self, select: bool = False, keystring: str = None) -> None:
         """Follow a hint.
 
@@ -1074,6 +1079,9 @@ class WordHinter:
             except OSError as e:
                 error = "Word hints requires reading the file at {}: {}"
                 raise HintingError(error.format(dictionary, str(e)))
+            except UnicodeDecodeError as e:
+                error = "Word hints expects the file at {} to be encoded as UTF-8: {}"
+                raise HintingError(error.format(dictionary, str(e)))
 
     def extract_tag_words(
             self, elem: webelem.AbstractWebElement
