@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2021 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -15,7 +15,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with qutebrowser.  If not, see <http://www.gnu.org/licenses/>.
+# along with qutebrowser.  If not, see <https://www.gnu.org/licenses/>.
 
 """Backend-independent qute://* code.
 
@@ -31,28 +31,21 @@ import time
 import textwrap
 import urllib
 import collections
-import base64
-import typing
-from typing import TypeVar, Callable, Union, Tuple
+import secrets
+from typing import TypeVar, Callable, Dict, List, Optional, Union, Sequence, Tuple
 
-try:
-    import secrets
-except ImportError:
-    # New in Python 3.6
-    secrets = None  # type: ignore
-
-from PyQt5.QtCore import QUrlQuery, QUrl, qVersion
+from PyQt5.QtCore import QUrlQuery, QUrl
 
 import qutebrowser
 from qutebrowser.browser import pdfjs, downloads, history
-from qutebrowser.config import config, configdata, configexc, configdiff
+from qutebrowser.config import config, configdata, configexc
 from qutebrowser.utils import (version, utils, jinja, log, message, docutils,
-                               objreg, urlutils, standarddir)
+                               resources, objreg, standarddir)
+from qutebrowser.misc import guiprocess
 from qutebrowser.qt import sip
 
 
 pyeval_output = ":pyeval was never called"
-spawn_output = ":spawn was never called"
 csrf_token = None
 
 
@@ -98,8 +91,8 @@ class Redirect(Exception):
 
 
 # Return value: (mimetype, data) (encoded as utf-8 if a str is returned)
-_Handler = TypeVar('_Handler',
-                   bound=Callable[[QUrl], Tuple[str, Union[str, bytes]]])
+_HandlerRet = Tuple[str, Union[str, bytes]]
+_Handler = TypeVar('_Handler', bound=Callable[[QUrl], _HandlerRet])
 
 
 class add_handler:  # noqa: N801,N806 pylint: disable=invalid-name
@@ -110,22 +103,22 @@ class add_handler:  # noqa: N801,N806 pylint: disable=invalid-name
         _name: The 'foo' part of qute://foo
     """
 
-    def __init__(self, name):
+    def __init__(self, name: str) -> None:
         self._name = name
-        self._function = None  # type: typing.Optional[typing.Callable]
+        self._function: Optional[Callable] = None
 
     def __call__(self, function: _Handler) -> _Handler:
         self._function = function
         _HANDLERS[self._name] = self.wrapper
         return function
 
-    def wrapper(self, *args, **kwargs):
+    def wrapper(self, url: QUrl) -> _HandlerRet:
         """Call the underlying function."""
         assert self._function is not None
-        return self._function(*args, **kwargs)
+        return self._function(url)
 
 
-def data_for_url(url):
+def data_for_url(url: QUrl) -> Tuple[str, bytes]:
     """Get the data to show for the given URL.
 
     Args:
@@ -134,14 +127,15 @@ def data_for_url(url):
     Return:
         A (mimetype, data) tuple.
     """
-    norm_url = url.adjusted(QUrl.NormalizePathSegments |
-                            QUrl.StripTrailingSlash)
+    norm_url = url.adjusted(
+        QUrl.NormalizePathSegments |  # type: ignore[arg-type]
+        QUrl.StripTrailingSlash)
     if norm_url != url:
         raise Redirect(norm_url)
 
     path = url.path()
     host = url.host()
-    query = urlutils.query_string(url)
+    query = url.query()
     # A url like "qute:foo" is split as "scheme:path", not "scheme:host".
     log.misc.debug("url: {}, path: {}, host {}".format(
         url.toDisplayString(), path, host))
@@ -176,12 +170,13 @@ def data_for_url(url):
     if mimetype == 'text/html' and isinstance(data, str):
         # We let handlers return HTML as text
         data = data.encode('utf-8', errors='xmlcharrefreplace')
+    assert isinstance(data, bytes)
 
     return mimetype, data
 
 
 @add_handler('bookmarks')
-def qute_bookmarks(_url):
+def qute_bookmarks(_url: QUrl) -> _HandlerRet:
     """Handler for qute://bookmarks. Display all quickmarks / bookmarks."""
     bookmarks = sorted(objreg.get('bookmark-manager').marks.items(),
                        key=lambda x: x[1])  # Sort by title
@@ -196,10 +191,9 @@ def qute_bookmarks(_url):
 
 
 @add_handler('tabs')
-def qute_tabs(_url):
+def qute_tabs(_url: QUrl) -> _HandlerRet:
     """Handler for qute://tabs. Display information about all open tabs."""
-    tabs = collections.defaultdict(
-        list)  # type: typing.Dict[str, typing.List[typing.Tuple[str, str]]]
+    tabs: Dict[str, List[Tuple[str, str]]] = collections.defaultdict(list)
     for win_id, window in objreg.window_registry.items():
         if sip.isdeleted(window):
             continue
@@ -217,7 +211,10 @@ def qute_tabs(_url):
     return 'text/html', src
 
 
-def history_data(start_time, offset=None):
+def history_data(
+        start_time: float,
+        offset: int = None
+) -> Sequence[Dict[str, Union[str, int]]]:
     """Return history data.
 
     Arguments:
@@ -240,7 +237,7 @@ def history_data(start_time, offset=None):
 
 
 @add_handler('history')
-def qute_history(url):
+def qute_history(url: QUrl) -> _HandlerRet:
     """Handler for qute://history. Display and serve history."""
     if url.path() == '/data':
         q_offset = QUrlQuery(url).queryItemValue("offset")
@@ -266,7 +263,7 @@ def qute_history(url):
 
 
 @add_handler('javascript')
-def qute_javascript(url):
+def qute_javascript(url: QUrl) -> _HandlerRet:
     """Handler for qute://javascript.
 
     Return content of file given as query parameter.
@@ -274,91 +271,108 @@ def qute_javascript(url):
     path = url.path()
     if path:
         path = "javascript" + os.sep.join(path.split('/'))
-        return 'text/html', utils.read_file(path, binary=False)
+        return 'text/html', resources.read_file(path)
     else:
         raise UrlInvalidError("No file specified")
 
 
 @add_handler('pyeval')
-def qute_pyeval(_url):
+def qute_pyeval(_url: QUrl) -> _HandlerRet:
     """Handler for qute://pyeval."""
     src = jinja.render('pre.html', title='pyeval', content=pyeval_output)
     return 'text/html', src
 
 
-@add_handler('spawn-output')
-def qute_spawn_output(_url):
-    """Handler for qute://spawn-output."""
-    src = jinja.render('pre.html', title='spawn output', content=spawn_output)
+@add_handler('process')
+def qute_process(url: QUrl) -> _HandlerRet:
+    """Handler for qute://process."""
+    path = url.path()[1:]
+    try:
+        pid = int(path)
+    except ValueError:
+        raise UrlInvalidError(f"Invalid PID {path}")
+
+    try:
+        proc = guiprocess.all_processes[pid]
+    except KeyError:
+        raise NotFoundError(f"No process {pid}")
+
+    if proc is None:
+        raise NotFoundError(f"Data for process {pid} got cleaned up.")
+
+    src = jinja.render('process.html', title=f'Process {pid}', proc=proc)
     return 'text/html', src
 
 
 @add_handler('version')
 @add_handler('verizon')
-def qute_version(_url):
+def qute_version(_url: QUrl) -> _HandlerRet:
     """Handler for qute://version."""
     src = jinja.render('version.html', title='Version info',
-                       version=version.version(),
+                       version=version.version_info(),
                        copyright=qutebrowser.__copyright__)
     return 'text/html', src
 
 
-@add_handler('plainlog')
-def qute_plainlog(url):
-    """Handler for qute://plainlog.
-
-    An optional query parameter specifies the minimum log level to print.
-    For example, qute://log?level=warning prints warnings and errors.
-    Level can be one of: vdebug, debug, info, warning, error, critical.
-    """
-    if log.ram_handler is None:
-        text = "Log output was disabled."
-    else:
-        level = QUrlQuery(url).queryItemValue('level')
-        if not level:
-            level = 'vdebug'
-        text = log.ram_handler.dump_log(html=False, level=level)
-    src = jinja.render('pre.html', title='log', content=text)
-    return 'text/html', src
-
-
 @add_handler('log')
-def qute_log(url):
+def qute_log(url: QUrl) -> _HandlerRet:
     """Handler for qute://log.
 
-    An optional query parameter specifies the minimum log level to print.
+    There are three query parameters:
+
+    - level: The minimum log level to print.
     For example, qute://log?level=warning prints warnings and errors.
     Level can be one of: vdebug, debug, info, warning, error, critical.
+
+    - plain: If given (and not 'false'), plaintext is shown.
+
+    - logfilter: A filter string like the --logfilter commandline argument
+      accepts.
     """
+    query = QUrlQuery(url)
+    plain = (query.hasQueryItem('plain') and
+             query.queryItemValue('plain').lower() != 'false')
+
     if log.ram_handler is None:
-        html_log = None
+        content = "Log output was disabled." if plain else None
     else:
-        level = QUrlQuery(url).queryItemValue('level')
+        level = query.queryItemValue('level')
         if not level:
             level = 'vdebug'
-        html_log = log.ram_handler.dump_log(html=True, level=level)
 
-    src = jinja.render('log.html', title='log', content=html_log)
+        filter_str = query.queryItemValue('logfilter')
+
+        try:
+            logfilter = (log.LogFilter.parse(filter_str, only_debug=False)
+                         if filter_str else None)
+        except log.InvalidLogFilterError as e:
+            raise UrlInvalidError(e)
+
+        content = log.ram_handler.dump_log(html=not plain,
+                                           level=level, logfilter=logfilter)
+
+    template = 'pre.html' if plain else 'log.html'
+    src = jinja.render(template, title='log', content=content)
     return 'text/html', src
 
 
 @add_handler('gpl')
-def qute_gpl(_url):
+def qute_gpl(_url: QUrl) -> _HandlerRet:
     """Handler for qute://gpl. Return HTML content as string."""
-    return 'text/html', utils.read_file('html/license.html')
+    return 'text/html', resources.read_file('html/license.html')
 
 
-def _asciidoc_fallback_path(html_path):
+def _asciidoc_fallback_path(html_path: str) -> Optional[str]:
     """Fall back to plaintext asciidoc if the HTML is unavailable."""
     path = html_path.replace('.html', '.asciidoc')
     try:
-        return utils.read_file(path)
+        return resources.read_file(path)
     except OSError:
         return None
 
 
 @add_handler('help')
-def qute_help(url):
+def qute_help(url: QUrl) -> _HandlerRet:
     """Handler for qute://help."""
     urlpath = url.path()
     if not urlpath or urlpath == '/':
@@ -372,14 +386,14 @@ def qute_help(url):
     path = 'html/doc/{}'.format(urlpath)
     if not urlpath.endswith('.html'):
         try:
-            bdata = utils.read_file(path, binary=True)
+            bdata = resources.read_file_binary(path)
         except OSError as e:
             raise SchemeOSError(e)
         mimetype = utils.guess_mimetype(urlpath)
         return mimetype, bdata
 
     try:
-        data = utils.read_file(path)
+        data = resources.read_file(path)
     except OSError:
         asciidoc = _asciidoc_fallback_path(path)
 
@@ -407,7 +421,7 @@ def qute_help(url):
         return 'text/html', data
 
 
-def _qute_settings_set(url):
+def _qute_settings_set(url: QUrl) -> _HandlerRet:
     """Handler for qute://settings/set."""
     query = QUrlQuery(url)
     option = query.queryItemValue('option', QUrl.FullyDecoded)
@@ -429,7 +443,7 @@ def _qute_settings_set(url):
 
 
 @add_handler('settings')
-def qute_settings(url):
+def qute_settings(url: QUrl) -> _HandlerRet:
     """Handler for qute://settings. View/change qute configuration."""
     global csrf_token
 
@@ -442,12 +456,7 @@ def qute_settings(url):
     # Requests to qute://settings/set should only be allowed from
     # qute://settings. As an additional security precaution, we generate a CSRF
     # token to use here.
-    if secrets:
-        csrf_token = secrets.token_urlsafe()
-    else:
-        # On Python < 3.6, from secrets.py
-        token = base64.urlsafe_b64encode(os.urandom(32))
-        csrf_token = token.rstrip(b'=').decode('ascii')
+    csrf_token = secrets.token_urlsafe()
 
     src = jinja.render('settings.html', title='settings',
                        configdata=configdata,
@@ -457,7 +466,7 @@ def qute_settings(url):
 
 
 @add_handler('bindings')
-def qute_bindings(_url):
+def qute_bindings(_url: QUrl) -> _HandlerRet:
     """Handler for qute://bindings. View keybindings."""
     bindings = {}
     defaults = config.val.bindings.default
@@ -475,7 +484,7 @@ def qute_bindings(_url):
 
 
 @add_handler('back')
-def qute_back(url):
+def qute_back(url: QUrl) -> _HandlerRet:
     """Handler for qute://back.
 
     Simple page to free ram / lazy load a site, goes back on focusing the tab.
@@ -487,22 +496,14 @@ def qute_back(url):
 
 
 @add_handler('configdiff')
-def qute_configdiff(url):
+def qute_configdiff(_url: QUrl) -> _HandlerRet:
     """Handler for qute://configdiff."""
-    if url.path() == '/old':
-        try:
-            return 'text/html', configdiff.get_diff()
-        except OSError as e:
-            error = (b'Failed to read old config: ' +
-                     str(e.strerror).encode('utf-8'))
-            return 'text/plain', error
-    else:
-        data = config.instance.dump_userconfig().encode('utf-8')
-        return 'text/plain', data
+    data = config.instance.dump_userconfig().encode('utf-8')
+    return 'text/plain', data
 
 
 @add_handler('pastebin-version')
-def qute_pastebin_version(_url):
+def qute_pastebin_version(_url: QUrl) -> _HandlerRet:
     """Handler that pastebins the version string."""
     version.pastebin_version()
     return 'text/plain', b'Paste called.'
@@ -515,7 +516,7 @@ def _pdf_path(filename: str) -> str:
 
 
 @add_handler('pdfjs')
-def qute_pdfjs(url: QUrl):
+def qute_pdfjs(url: QUrl) -> _HandlerRet:
     """Handler for qute://pdfjs.
 
     Return the pdf.js viewer or redirect to original URL if the file does not
@@ -566,14 +567,10 @@ def qute_pdfjs(url: QUrl):
 
 
 @add_handler('warning')
-def qute_warning(url):
+def qute_warning(url: QUrl) -> _HandlerRet:
     """Handler for qute://warning."""
     path = url.path()
-    if path == '/old-qt':
-        src = jinja.render('warning-old-qt.html',
-                           title='Old Qt warning',
-                           qt_version=qVersion())
-    elif path == '/webkit':
+    if path == '/webkit':
         src = jinja.render('warning-webkit.html',
                            title='QtWebKit backend warning')
     elif path == '/sessions':
@@ -584,3 +581,15 @@ def qute_warning(url):
     else:
         raise NotFoundError("Invalid warning page {}".format(path))
     return 'text/html', src
+
+
+@add_handler('resource')
+def qute_resource(url: QUrl) -> _HandlerRet:
+    """Handler for qute://resource."""
+    path = url.path().lstrip('/')
+    mimetype = utils.guess_mimetype(path, fallback=True)
+    try:
+        data = resources.read_file_binary(path)
+    except FileNotFoundError as e:
+        raise NotFoundError(str(e))
+    return mimetype, data
